#!/bin/bash

#Check user privilege
user=`whoami`

if [ "$user" != "root" ] ;
then
        echo "User does not have root priviledge "
        exit
fi


#Check for the installation log file
installLogFile=/var/log/PMC-Sierra_maxRAID.log
if [ ! -e /etc/PMC-Sierra/maxRAID.conf ] ; then
	if [ -e $installLogFile ] ; then
	        rm -rf $installLogFile
	fi
fi


#Check if any other copy of the installer is running
echo Checking if any other copy of the installer is running >> $installLogFile
running=0
running=`ps -ef | grep Install_pmc_maxRAID.sh | wc -l`
if [ $running -gt 3 ] ;
then
        echo Another installer is already running
        echo Another installer is already running >> $installLogFile
	read -p "Press any key to exit..." anykey
        exit
fi


#Check OS level
echo Checking OS level >> $installLogFile
#Get the list of OS levels from maxRAID.conf file
oldstr=""
newstr=""
while read newstr ; do
	if [ "$oldstr" == "[SupportedOSLevel]" ] ; then
		break
	else
		oldstr=$newstr
	fi
done < scripts/maxRAID.conf

#Compare the OS with the list of supported OSs
echo Comparing the OS with the list of supported OSs >> $installLogFile
cat /etc/*release > /tmp/release
found=0
i=1
os=`echo $newstr | cut -d: -f$i`
preos=""
while [ "$os" != "" ]
do
	preos=$os
	while read line ; do
		if [ `echo $line | grep "$os" | wc -l` -eq 1 ] ; then
			found=1
			break
		fi
	done < /tmp/release
	if [ $found -eq 1 ] ; then
		break
	fi
	i=`expr $i + 1`
	os=`echo $newstr | cut -d: -f$i`
	if [ "$os" == "$preos" ] ; then
		break
	fi
done
rm -rf /tmp/release

if [ $found -eq 0 ] ;
then
	echo ""
	echo !!!!!!!!!!!!!!!!!!!!WARNING!!!!!!!!!!!!!!!!!!!!
	echo ""
	echo maxRAID Host Software Application is not supported for this operating system >> $installLogFile
	echo maxRAID Host Software Application is not supported for this operating system.
	echo -n Press any key to continue the installation or press 0 to abort the installation....
	read anykey
	if [ "$anykey" == "0" ] ; then
		echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
		exit
	fi
fi


#Retrieve the installer size
echo Calculating the size of the installer >> $installLogFile
for size in `du rpms/ | cut -f1`
do
#Nothing to do here
nothing=0
done
#'$size' contains size of the rpms. So adding the extra KByte which will equate the size with the actual size
installerSize=`expr $size + 127776`

#Check if driver is there in the system
if [ ! -e /etc/PMC-Sierra/maxRAID.conf ] ; then
	echo Checking if pmcraid driver is present in the system >> $installLogFile
	if [ `lsmod | grep pmcraid | wc -l` -eq 0 ] && [ `rpm -qa | grep pmcraid | wc -l` -eq 0 ] && [ `find /lib/modules/\`uname -r\`/ -name "pmcraid.ko" | wc -l` -eq 0 ]; then
		echo ""
		echo !!!!!!!!!!!!!!!!!!!!WARNING!!!!!!!!!!!!!!!!!!!!
		echo ""
		echo pmcraid driver is not installed in the system. Few components may not work properly.
		echo pmcraid driver is not installed in the system >> $installLogFile
		echo -n Press any key to continue the installation or press 0 to abort the installation....
		read anykey
		if [ "$anykey" == "0" ] ; then
			echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
			exit
		fi
	fi
fi

#Adding entry to /etc/hosts file
echo Adding entry to /etc/hosts file >> $installLogFile
host=`hostname`
if [ "`cat /etc/hosts | grep 127.0.0.1`" == "" ];then
             echo -e "127.0.0.1  \t localhost  \t $host" >> /etc/hosts

elif [ "`cat /etc/hosts | grep 127.0.0.1 | awk '{print $3}'`" == "" ];then
		sed -e '/127.0.0.1/s/localhost/&    '${host}'/' /etc/hosts > /tmp/host_new
		cat /tmp/host_new > /etc/hosts
		rm -rf /tmp/host_new
fi


#Check if user want the installer to run in silent mode

if [ "$1" == "--quiet" ] ;
then
	echo Installer is running in silent mode >> $installLogFile
	#Check the size (in KByte) of /opt directory
	echo Calculating the size of the installed directory >> $installLogFile
	if [ "`df /opt | awk '{print $4}' | sed -n '2p;2q'`" == "" ];then
                available=`df /opt | awk '{print $3}' | sed -n '3p;3q'`
        else
                available=`df /opt | awk '{print $4}' | sed -n '2p;2q'`
        fi

	if [ $available -lt $installerSize ] ; then 
		echo There is not enough space in /opt. 
		echo There is not enough space in /opt >> $installLogFile
		echo Exiting installation...
		echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
		exit
	fi

	#Check if the product is already installed
	echo Checking if the product is already installed >> $installLogFile
	sh scripts/Verify.sh -p
	status=$?

	if [ $status -eq 0 ] ;
	then
		#Product is not installed
		echo Saving the port numbers >> $installLogFile
		CIMServer_httpPort=5988
                CIMServer_httpsPort=5989
                WebServer_Port=8443
                echo CIMServer_httpPort=$CIMServer_httpPort > /tmp/pmc_custom_ports
                echo CIMServer_httpsPort=$CIMServer_httpsPort >>  /tmp/pmc_custom_ports
                echo WebServer_Port=$WebServer_Port >> /tmp/pmc_custom_ports

                #Retrieving the list of installed components
                echo Retrieving the list of installed components >> $installLogFile
                newStr=""
                oldStr=""
                compList=""
                for newStr in `cat scripts/maxRAID.conf`
                do
  	              if [ "$oldStr" == "[AvailableComponents]" ] ; then
        	              compList=$newStr
			      break
                      else
                              oldStr=$newStr
                      fi
                done
		
		sh scripts/RunRPM.sh -i all 
		status=$?
	else
		echo The product is already installed.
		echo Checking if the installer version is higher than the installed one >> $installLogFile	
		sh scripts/Verify.sh -v
		status=$?
		if [ $status -eq 255 ] ; then
			echo A higher version of this product is already installed in this system
			echo A higher version of this product is already installed in this system >> $installLogFile
			echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
			exit
		else
			CIMServer_httpPort=5988
        	        CIMServer_httpsPort=5989
	                WebServer_Port=8443
			echo CIMServer_httpPort=$CIMServer_httpPort > /tmp/pmc_custom_ports
	        	echo CIMServer_httpsPort=$CIMServer_httpsPort >>  /tmp/pmc_custom_ports
        		echo WebServer_Port=$WebServer_Port >> /tmp/pmc_custom_ports

			if [ $status -eq 1 ] ; then
				sh scripts/Upgrade.sh --quiet
				echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
				exit
			else
				cp /etc/PMC-Sierra/maxRAID.conf /tmp/
				sh scripts/Configure.sh -u --quiet
				sh scripts/Remove.sh
				
				#Retrieving the list of installed components
    	                        echo Retrieving the list of installed components >> $installLogFile
				newStr=""
        	                oldStr=""
                	        compList=""
                        	for newStr in `cat /tmp/maxRAID.conf`
                             	do
                                	if [ "$oldStr" == "[InstalledComponent]" ] ; then
                                        	compList=$newStr
                                             	break
                                     	else
                                                oldStr=$newStr
                                     	fi
                             	done
				rm -rf /tmp/maxRAID.conf
				#Installing the component selected
			        echo Installing the selected components >> $installLogFile
				i=1
                             	oldComp=""
                             	comp=`echo $compList | cut -d: -f$i`
                             	while [ "$comp" != "" ]
	                             do
        	                     	if [ "$comp" == "$oldComp" ] ; then
                	                	break
                        	        fi

				     sh scripts/RunRPM.sh -i $comp 
                                     status=$?
				     if [ $? -ne 0 ] ; then
                                             echo Installation failed.
					     echo Installation failed >> $installLogFile
                                             sh scripts/Rollback.sh 
                                             rm -f /tmp/maxRAID.conf
                                             exit
                                     fi
                                     i=`expr $i + 1`
                                     oldComp=$comp
                                     comp=`echo $compList | cut -d: -f$i`
                                done

				#sh scripts/RunRPM.sh -i all
				#status=$?
			fi
		fi
	fi

	if [ $status -eq 0 ] ;
	then
		echo Configuring the product >> $installLogFile
		sh scripts/Configure.sh -c /opt
		status=$?
		echo " " >> /etc/PMC-Sierra/maxRAID.conf
		echo [InstalledComponent] >> /etc/PMC-Sierra/maxRAID.conf
		echo $compList >> /etc/PMC-Sierra/maxRAID.conf

		if [ $status -ne 0 ] ;
		then
			echo Configuration Failed 
			echo Configuration Failed...Rolling back the changes >> $installLogFile
			sh scripts/Rollback.sh
			echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
			exit
		fi
	else
		echo Installation failed 
		echo Installation failed...Rolling back the changes >> $installLogFile
		sh scripts/Rollback.sh /opt
		echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
		exit
	fi

	if [ $status -eq 0 ] ;
	then
		echo Product is installed successfully. See the log in /var/log/PMC-Sierra_maxRAID.log for details.
		echo Product is installed successfully >> $installLogFile
	fi
	exit

elif [ "$1" != "" ] ;
then
	echo "Install_pmc_maxRAID.sh: invalid usage '$1'"
	echo Usage:-
	echo 1. sh Install_pmc_maxRAID.sh
	echo 2. sh Install_pmc_maxRAID.sh --quiet
	echo Exiting installation
	exit
fi

#Check if the product is already installed
echo Checking if the product is already installed >> $installLogFile
sh scripts/Verify.sh -p
status=$?
if [ $status -eq 0 ] ;
then
	#Prompt user for license agreement
	cat license.txt | more
	EULA=""
	while [ "$EULA" != "y" ] && [ "$EULA" != "Y" ] && [ "$EULA" != "n" ] && [ "$EULA" != "N" ]
	do
		echo -n "I accept the terms in the license Agreement (Y)es/(N)o: "
		read EULA
	done

	if [ "$EULA" == "N" ] || [ "$EULA" == "n" ] ;
	then
		echo User Did not accept the license agreement
		exit
	fi

        echo maxRAID Host Software Application will be installed at /opt/PMC-Sierra/maxRAID
        echo  -n Would you like to change the default location Y/N:
        read defaultChoice
        installPath=""
	if [ "$defaultChoice" == "Y" ] || [ "$defaultChoice" == "y" ] ;
        then
                echo -n Enter new location:
                read installPath
                if [ -d $installPath ] ;
                then
			#Check the permission of the directory
			#Check whether the specified directory contains root dir
			echo Checking permission of user selected directory >> $installLogFile
			if [ "$installPath" == "/" ] ;
                        then
                                echo The specific path contains root directory
				echo The specific path contains root directory >> $installLogFile
				echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
                                exit
                        fi
                else
                        echo The specified location $installPath does not exist
			echo The specified location $installPath does not exist >> $installLogFile
			echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
                        exit
                fi
        elif [ "$defaultChoice" == "N" ] || [ "$defaultChoice" == "n" ];
	then
        	echo "Proceeding with default location.."
		echo Proceeding with default location for installation >> $installLogFile
	else
		echo "Wrong choice...Exiting Installation"
		exit
        fi

	#Check for available space
	if [ "$installPath" == "" ] ; then
		echo Checking the size of the installed directory >> $installLogFile
		if [ "`df /opt | awk '{print $4}' | sed -n '2p;2q'`" == "" ];then
	                available=`df /opt | awk '{print $3}' | sed -n '3p;3q'`
        	else
                	available=`df /opt | awk '{print $4}' | sed -n '2p;2q'`
        	fi

		if [ $available -lt $installerSize ] ; then
			echo There is not enough space in /opt.
			echo Exiting installation..
			echo There is not enough space in /opt....Exiting installation >> $installLogFile
			echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
			exit
		fi
	else
		echo Checking the size of the installed directory
		echo Checking the size of the installed directory >> $installLogFile
		if [ "`df /opt | awk '{print $4}' | sed -n '2p;2q'`" == "" ];then
	                available=`df /opt | awk '{print $3}' | sed -n '3p;3q'`
        	else
                	available=`df /opt | awk '{print $4}' | sed -n '2p;2q'`
        	fi

		if [ $available -lt $installerSize ] ; then
			echo There is not enough space in $installPath.
			echo Exiting installation..
			echo There is not enough space in $installPath...Exiting installation >> $installLogFile
			echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
			exit
                fi
	fi
else
	sh scripts/Verify.sh -v
	status=$?
	echo Checking the installer version
	echo Checking the installer version >> $installLogFile
	if [ $status -eq 0 ] ; then
		sh scripts/Modify.sh 
	elif [ $status -eq 1 ]
	then
		sh scripts/Upgrade.sh 
		echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
		exit
	else
		echo A higher version of this product is already installed in this system
		echo A higher version of this product is already installed in this system >> $installLogFile
	fi
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
fi


echo 1. Complete Installation
echo 2. Custom Installation
echo -n [Press 0 to Quit] : 
read com_or_cus

if [ "$com_or_cus" == "0" ] ;
then
	echo Exiting installation
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
fi

if [ "$installPath" == "" ] ; then
	installPath="/opt"
fi

components=""

case $com_or_cus in
1)
	echo Validating the chosen ports >> $installLogFile
	validate_port()
	{
        if [ "`netstat -an | grep -w $2`" == "" -a $2 -ne $3 -a $2 -ne $4 ];then
                PORT_NUMBER=$2
        else
                echo Selected port is not free...giving a free port
                for ((  i = 49152  ;  i <= 65535;  i++  ))
                do
                        if [ "`netstat -an | grep -w $i`" == "" ];then
                                if [ $i -ne $2 -a $i -ne $3 -a $i -ne $4 ];then
                                        PORT_NUMBER=$i
                                        break
                                fi
                        fi
                done
        fi
	}

	CIMServer_httpPort=5988
	CIMServer_httpsPort=5989
	WebServer_Port=8443

	echo Default CIMServer_httpPort is 5988
	echo Do you want to change this port\?
	echo -n "Press \"Y\" to change and \"N\" to continue with the default port:"
	read port_change
	if [ "$port_change" == "Y" ] || [ "$port_change" == "y" ];then
        	PORT="CIMServer_httpPort"
	        echo Enter port number
        	read PORT_NUMBER
	else
        	PORT="CIMServer_httpPort"
	        PORT_NUMBER=$CIMServer_httpPort
	fi
        validate_port "$PORT" "$PORT_NUMBER" 0 0
        CIMServer_httpPort=$PORT_NUMBER
	
	echo Default CIMServer_httpsPort is 5989
	echo Do you want to change this port\?
	echo -n "Press \"Y\" to change and \"N\" to continue with the default port:"
	read port_change
	if [ "$port_change" == "Y" ] || [ "$port_change" == "y" ];then
        	PORT="CIMServer_httpsPort"
	        echo Enter port number
        	read PORT_NUMBER
	else
        	PORT="CIMServer_httpsPort"
	        PORT_NUMBER=$CIMServer_httpsPort
	fi
        validate_port "$PORT" "$PORT_NUMBER" $CIMServer_httpPort 0
        CIMServer_httpsPort=$PORT_NUMBER

	echo Default WebServer_Port is 8443
	echo Do you want to change this port\?
	echo -n "Press \"Y\" to change and \"N\" to continue with the default port:"
	read port_change
	if [ "$port_change" == "Y" ] || [ "$port_change" == "y" ];then
	        PORT="WebServer_Port"
        	echo Enter port number
	        read PORT_NUMBER
	else
        	PORT="WebServer_Port"
	        PORT_NUMBER=$WebServer_Port
	fi
        validate_port "$PORT" "$PORT_NUMBER" $CIMServer_httpPort $CIMServer_httpsPort
        WebServer_Port=$PORT_NUMBER

	echo Selected ports are
	echo [CustomPorts] > /tmp/pmc_custom_ports
	echo CIMServer_httpPort=$CIMServer_httpPort | tee -a /tmp/pmc_custom_ports
	echo CIMServer_httpsPort=$CIMServer_httpsPort | tee -a /tmp/pmc_custom_ports
	echo WebServer_Port=$WebServer_Port | tee -a /tmp/pmc_custom_ports

	sh scripts/RunRPM.sh -i all $installPath
	status=$?
        if [ $status -ne 0 ] ; then
                echo Installation failed...
                echo Installation failed...Rolling back changes >> $installLogFile
		sh scripts/Rollback.sh $installPath
		echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
                exit
        fi
	components="CLI:Extended_Manageability"
	;;

2)
    echo "Please  select from the options...(use space between two choices)"
    echo         "1. CLI Feature"
    echo         "2. Management Software Feature"
    echo         -n [Press 0 to quit] :
    read customChoice

    if [ "$customChoice" == "0" ] ;
    then
	echo Exiting Installation
	echo Custom choice is wrong...exiting Installation >> $installLogFile
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
    fi

    #Validate the choice
    _status=1
    i=1
    temp=`echo $customChoice | cut -d' ' -f$i`
    oldTemp=$temp
    while [ "$temp" != "" ]
    do
	if [ "$temp" != "1" ] && [ "$temp" != "2" ] ; then
                _status=0
                break
        fi
        i=`expr $i + 1`
        temp=`echo $customChoice | cut -d' ' -f$i`
	if [ "$oldTemp" == "$temp" ] ; then
		break
	fi
    done
    if [ $_status -eq 0 ] ; then
	echo One or more input options are wrong. Exiting installation.
	echo One or more input options are wrong. Exiting installation >> $installLogFile
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
    fi

    i=1
    choice=`echo $customChoice | cut -d' ' -f$i`
    oldChoice=$choice
    while [ "$choice" != "" ]
    do
	case $choice in
		1)
		sh scripts/RunRPM.sh -i CLI $installPath
		status=$?
		if [ "$components" == "" ] ; then
			components="CLI"
		else
			components=$components":CLI"
		fi
		;;
		2)
                
		echo Validating the selected ports >> $installLogFile  
		validate_port()
		{
        		if [ "`netstat -an | grep -w $2`" == "" -a $2 -ne $3 -a $2 -ne $4 ];then
                		PORT_NUMBER=$2
		        else
        	        	echo Selected port is not free...giving a free port
	        	        for ((  i = 49152  ;  i <= 65535;  i++  ))
        	        	do
	                	        if [ "`netstat -an | grep -w $i`" == "" ];then
        	                	        if [ $i -ne $2 -a $i -ne $3 -a $i -ne $4 ];then
                	                	        PORT_NUMBER=$i
                        	                	break
	                        	        fi
        	                	fi
	                	done
        		fi
		}

		CIMServer_httpPort=5988
		CIMServer_httpsPort=5989
		WebServer_Port=8443

		echo Default CIMServer_httpPort is 5988
		echo Do you want to change this port\?
		echo -n "Press \"Y\" to change and \"N\" to continue with the default port:"
		read port_change
		if [ "$port_change" == "Y" ] || [ "$port_change" == "y" ];then
		        PORT="CIMServer_httpPort"
		        echo Enter port number
		        read PORT_NUMBER
		else
		        PORT="CIMServer_httpPort"
		        PORT_NUMBER=$CIMServer_httpPort
		fi
	        validate_port "$PORT" "$PORT_NUMBER" 0 0
        	CIMServer_httpPort=$PORT_NUMBER
		
		echo Default CIMServer_httpsPort is 5989
		echo Do you want to change this port\?
		echo -n "Press \"Y\" to change and \"N\" to continue with the default port:"
		read port_change
		if [ "$port_change" == "Y" ] || [ "$port_change" == "y" ];then
		        PORT="CIMServer_httpsPort"
        		echo Enter port number
		        read PORT_NUMBER
		else
        		PORT="CIMServer_httpsPort"
		        PORT_NUMBER=$CIMServer_httpsPort
		fi
	        validate_port "$PORT" "$PORT_NUMBER" $CIMServer_httpPort 0
        	CIMServer_httpsPort=$PORT_NUMBER

		echo Default WebServer_Port is 8443
		echo Do you want to change this port\?
		echo -n "Press \"Y\" to change and \"N\" to continue with the default port:"
		read port_change
		if [ "$port_change" == "Y" ] || [ "$port_change" == "y" ];then
		        PORT="WebServer_Port"
        		echo Enter port number
		        read PORT_NUMBER
		else
		        PORT="WebServer_Port"
		        PORT_NUMBER=$WebServer_Port
		fi
	        validate_port "$PORT" "$PORT_NUMBER" $CIMServer_httpPort $CIMServer_httpsPort
        	WebServer_Port=$PORT_NUMBER

		echo Selected ports are
		echo [CustomPorts] > /tmp/pmc_custom_ports
		echo CIMServer_httpPort=$CIMServer_httpPort | tee -a /tmp/pmc_custom_ports
		echo CIMServer_httpsPort=$CIMServer_httpsPort | tee -a /tmp/pmc_custom_ports
		echo WebServer_Port=$WebServer_Port | tee -a /tmp/pmc_custom_ports


		sh scripts/RunRPM.sh -i Extended_Manageability $installPath
                status=$?
                if [ "$components" == "" ] ; then
	                components="Extended_Manageability"
                else
	                components=$components":Extended_Manageability"
                fi
		;;
    	esac

	if [ $status -ne 0 ] ; then
		echo Installation failed...
		echo Installation failed...Rolling back the changes >> $installLogFile
		sh scripts/Rollback.sh $installPath
		echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
		exit
	fi

	i=`expr $i + 1`
	choice=`echo $customChoice | cut -d' ' -f$i`
	if [ "$oldChoice" == "$choice" ] ; then
		break
	fi
    done
    ;;
*)
echo Wrong choice
echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
exit
;;
esac

source scripts/Configure.sh -c $installPath
status=$?
if [ $status -ne 0 ] ; then
	echo Configuration failed.
	echo Configuration failed...Rolling back the changes >> $installLogFile
	sh scripts/Rollback.sh $installPath
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
fi

echo " " >> /etc/PMC-Sierra/maxRAID.conf
echo [InstalledComponent] >> /etc/PMC-Sierra/maxRAID.conf
echo $components >> /etc/PMC-Sierra/maxRAID.conf


if [ $status -eq 0 ] ;
then
	echo Product is installed successfully.
	echo Product is installed successfully >> $installLogFile
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
else
	echo Product installation failed
	echo Product installation failed >> $installLogFile
	sh scripts/Rollback.sh $installPath
	echo See the log in /var/log/PMC-Sierra_maxRAID.log for details.
	exit
fi
