#!/bin/bash

installLogFile=/var/log/PMC-Sierra_maxRAID.log

installCmpnt() {

_cmpnt=""
case $1 in
#Locate the rpm component
        "CLI")
	if [ -e rpms/CLI*.rpm ] ; then
		cli=`ls rpms/CLI*.rpm`
		_cmpnt=$cli
		_prefix=$2
		_installPath=$3
	fi
        ;;
        "Extended_Manageability")
	if [ -e rpms/Extended_Manageability*.rpm ] ; then
		extended=`ls rpms/Extended_Manageability*.rpm`
		_cmpnt=$extended
		_prefix=$2
		_installPath=$3
	fi
        ;;

esac

#Install the component
if [ "$_cmpnt" != "" ] && [ -e $_cmpnt ] ; then
	if [ "$4" == "-i" ] ; then
		rpm -i --test $_cmpnt 2>| /tmp/warnings.txt
		status=$?
		if [ $status -eq 0 ] ; then
			echo Installing $_cmpnt >> $installLogFile
			echo -n Installing...
			echo $_cmpnt
			rpm -ivh $_prefix $_installPath $_cmpnt
			status=$?
			rm -rf /tmp/warnings.txt
			return $status
		fi
		rm -rf /tmp/warnings.txt
	else
		rpm -U --replacefiles --test $_cmpnt 2>| /tmp/warnings.txt
		status=$?
		if [ $status -eq 0 ] ; then
			echo Updating $_cmpnt >> $installLogFile
			echo -n Updating...
			echo $_cmpnt
			rpm -Uvh --replacefiles $_prefix $_installPath $_cmpnt
			status=$?
			rm -rf /tmp/warnings.txt
			return $status
		else
			rm -rf /tmp/warnings.txt
			echo Same version of $_cmpnt is already installed
			echo Same version of $_cmpnt is already installed >> $installLogFile
		fi
	fi
else
	echo $1 component is not present in the disk.
	return 255
fi
}

prefix=""
installPath=""
if [ "$1" == "-i" ] ; then
	if [ "$3" != "" ] ; then
		prefix="--prefix "
		installPath=$3
	else
		prefix="--prefix "
		installPath="/opt"
	fi
elif [ "$1" == "-u" ]
then
        _newStr=""
        _oldStr=""
#Get the installed path
        for _newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
        do
                if [ "$_oldStr" == "[InstalledPath]" ] ; then
                        break
                else
                        _oldStr=$_newStr
                fi
        done
        if [ "$3" != "" ] ; then
                prefix="--prefix "
                installPath=$_newStr
        else
                prefix="--prefix "
                installPath="/opt"
        fi
fi

case $2 in
	"all")
	found=0
	newStr=""
	oldStr=""
#Get the list of available components from the disk's maxRAID.conf file
	echo Get the list of available components >> $installLogFile
	for newStr in `cat scripts/maxRAID.conf`
	do
		if [ "$oldStr" == "[AvailableComponents]" ] ; then
			found=1
			break
		else
			oldStr=$newStr
		fi
	done
#Install all the components
	echo Installing all components >> $installLogFile
	if [ $found -eq 1 ] ; then
		i=1
		finish=0
		components=$newStr
		preCmpnt=""
		while [ $finish -eq 0 ]
		do
			cmpnt=`echo $components | cut -d: -f$i`
			if [ "$preCmpnt" == "$cmpnt" ] ; then
				break
			fi
			preCmpnt=$cmpnt
			if [ "$cmpnt" == "" ] ; then
				finish=1
				break
			fi
			installCmpnt $cmpnt $prefix $installPath $1
			status=$?
			if [ $status -eq 0 ] ; then echo Done.
			else
				echo Failed to install $cmpnt
				exit 255
			fi
			i=`expr $i + 1`
		done
	else
		exit 255
	fi
	;;

#Install CLI component
	"CLI")
	installCmpnt CLI $prefix $installPath $1
	status=$?
	if [ $status -eq 0 ] ; then echo Done.
	else
		echo Failed to install CLI
		echo Failed to install CLI >> $installLogFile
		exit 255
	fi
	;;

#Install Extended Manageability component
	"Extended_Manageability")
	installCmpnt Extended_Manageability $prefix $installPath $1
	status=$?
	if [ $status -eq 0 ] ; then echo Done.
	else
		echo Failed to install Extended Manageability component
		echo Failed to install Extended Manageability component >> $installLogFile
		exit 255
	fi
	;;
	*)
	echo Wrong choice
esac
