#!/bin/bash

installLogFile=/var/log/PMC-Sierra_maxRAID.log

if [ ! -e /etc/PMC-Sierra/maxRAID.conf ] ; then
	echo Configuration file /etc/PMC-Sierra/maxRAID.conf is missing.
	echo Configuration file /etc/PMC-Sierra/maxRAID.conf is missing >> $installLogFile
	echo Cannot upgrade
	echo Cannot upgrade >> $installLogFile
	rm -rf /tmp/pmc_installed
	exit
fi
if [ "$1" == "" ] ; then
	echo Lower version of the product is already installed
	echo Lower version of the product is already installed >> $installLogFile
	echo Do you want to upgrade the product?
	echo Enter \'yes\' to upgrade
	echo Enter \'no\' to quit

	read upgrade
	if [ "$upgrade" == "no" ] ; 
	then
		echo Product is not upgraded
		echo Product is not upgraded >> $installLogFile
		rm -rf /tmp/pmc_installed
		exit
	fi

	if [ "$upgrade" == "yes" ] ; 
	then
		echo Upgrading the product...
		echo Upgrading the product... >> $installLogFile
		echo Retrieving the list of installed components for upgrading the product >> $installLogFile
		newStr=""
		oldStr=""
		installedCmpnt=""
		for newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
		do
			if [ "$oldStr" == "[InstalledComponent]" ] ; then
				installedCmpnt=$newStr
				break
			else
				oldStr=$newStr
			fi
		done
		
		echo Retrieving the installed path for upgrading the product >> $installLogFile
		newStr=""
		oldStr=""
		installedPath=""
		for newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
		do	
        		if [ "$oldStr" == "[InstalledPath]" ] ; then
                		installedPath=$newStr
               	 		break
        		else
                		oldStr=$newStr
        		fi
		done

		newStr=""
		oldStr=""
		installedVer=""
		for newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
		do
        		if [ "$oldStr" == "[Version]" ] ; then
                		installedVer=$newStr
                		break
        		else
                		oldStr=$newStr
        		fi
		done

		i=1
		oldComp=""
		comp=`echo $installedCmpnt | cut -d: -f$i`
		while [ "$comp" != "" ]
		do
        		if [ "$comp" == "$oldComp" ] ; then
                		break
	        	fi
        		sh scripts/Configure.sh -uc $comp $installedPath
	        	if [ $? -ne 0 ] ; then
        	        	echo Unconfiguration failed.
				echo Unconfiguration failed...Rolling back the changes >> $installLogFile
	                	sh scripts/Rollback.sh
        	        	rm -rf /tmp/pmc_installed
				exit
	        	fi
	        	i=`expr $i + 1`
		        oldComp=$comp
        		comp=`echo $installedCmpnt | cut -d: -f$i`
		done
	

		i=1
		oldComp=""
		comp=`echo $installedCmpnt | cut -d: -f$i`
		while [ "$comp" != "" ]
		do
			if [ "$comp" == "$oldComp" ] ; then
				break
			fi
			sh scripts/RunRPM.sh -u $comp
			if [ $? -ne 0 ] ; then
				echo Installation failed.
				echo Installation failed...Rolling back the changes >> $installLogFile
				sh scripts/Rollback.sh
				rm -rf /tmp/pmc_installed
				exit
			fi
			i=`expr $i + 1`
			oldComp=$comp
			comp=`echo $installedCmpnt | cut -d: -f$i`
		done

		i=1
		oldComp=""
		comp=`echo $installedCmpnt | cut -d: -f$i`
		while [ "$comp" != "" ]
		do
        		if [ "$comp" == "$oldComp" ] ; then
               			break
        		fi
	        	sh scripts/Configure.sh -cc $comp $installedPath
        		if [ $? -ne 0 ] ; then
                		echo Configuration failed.
                		echo Configuration failed...Rolling back the changes >> $installLogFile	
				sh scripts/Rollback.sh
	                	rm -rf /tmp/pmc_installed
				exit
		        fi
        		i=`expr $i + 1`
	        	oldComp=$comp
        		comp=`echo $installedCmpnt | cut -d: -f$i`
		done


		newStr=""
		oldStr=""
		installerVer=""
		for newStr in `cat scripts/maxRAID.conf`
		do
        		if [ "$oldStr" == "[Version]" ] ; then
                		installerVer=$newStr
                		break
        		else
                		oldStr=$newStr
        		fi
		done

		echo Upgrading the version in the /etc/PMC-Sierra/maxRAID.conf file >> $installLogFile
		sed 's/'$installedVer'/'$installerVer'/' </etc/PMC-Sierra/maxRAID.conf >/etc/PMC-Sierra/maxRAID.conf.new
		rm -rf /etc/PMC-Sierra/maxRAID.conf
		mv /etc/PMC-Sierra/maxRAID.conf.new /etc/PMC-Sierra/maxRAID.conf

		echo Product is upgraded successfully.
		echo Product is upgraded successfully >> $installLogFile
	fi
else
if [ "$1" == "--quiet" ] ; then
	echo Upgrading the product...
	echo Upgrading the product in silent mode >> $installLogFile

                newStr=""
                oldStr=""
                installedCmpnt=""
                for newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
                do
                        if [ "$oldStr" == "[InstalledComponent]" ] ; then
                                installedCmpnt=$newStr
                                break
                        else
                                oldStr=$newStr
                        fi
                done

                newStr=""
                oldStr=""
                installedPath=""
                for newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
                do
                        if [ "$oldStr" == "[InstalledPath]" ] ; then
                                installedPath=$newStr
                                break
                        else
                                oldStr=$newStr
                        fi
                done

                newStr=""
                oldStr=""
                installedVer=""
                for newStr in `cat /etc/PMC-Sierra/maxRAID.conf`
                do
                        if [ "$oldStr" == "[Version]" ] ; then
                                installedVer=$newStr
                                break
                        else
                                oldStr=$newStr
                        fi
                done

		 i=1
                oldComp=""
                comp=`echo $installedCmpnt | cut -d: -f$i`
                while [ "$comp" != "" ]
                do
                        if [ "$comp" == "$oldComp" ] ; then
                                break
                        fi
                        sh scripts/Configure.sh -uc $comp $installedPath
                        if [ $? -ne 0 ] ; then
                                echo Unconfiguration failed.
                                echo Unconfiguration failed...Rolling back the changes >> $installLogFile
				sh scripts/Rollback.sh
                                rm -rf /tmp/pmc_installed
				exit
                        fi
                        i=`expr $i + 1`
                        oldComp=$comp
                        comp=`echo $installedCmpnt | cut -d: -f$i`
                done


                i=1
                oldComp=""
                comp=`echo $installedCmpnt | cut -d: -f$i`
                while [ "$comp" != "" ]
                do
                        if [ "$comp" == "$oldComp" ] ; then
                                break
                        fi
                        sh scripts/RunRPM.sh -u $comp
                        if [ $? -ne 0 ] ; then
                                echo Installation failed.
				echo Installation failed...Rolling back the changes >> $installLogFile
                                sh scripts/Rollback.sh
                                rm -rf /tmp/pmc_installed
				exit
                        fi
                        i=`expr $i + 1`
                        oldComp=$comp
                        comp=`echo $installedCmpnt | cut -d: -f$i`
                done
				
		i=1
                oldComp=""
                comp=`echo $installedCmpnt | cut -d: -f$i`
                while [ "$comp" != "" ]
                do
                        if [ "$comp" == "$oldComp" ] ; then
                                break
                        fi
                        sh scripts/Configure.sh -cc $comp $installedPath
                        if [ $? -ne 0 ] ; then
                                echo Configuration failed.
				echo Configuration failed...Rolling back changes >> $installLogFile
                                sh scripts/Rollback.sh
                                rm -rf /tmp/pmc_installed
				exit
                        fi
                        i=`expr $i + 1`
                        oldComp=$comp
                        comp=`echo $installedCmpnt | cut -d: -f$i`
                done


                newStr=""
                oldStr=""
                installerVer=""
                for newStr in `cat scripts/maxRAID.conf`
                do
                        if [ "$oldStr" == "[Version]" ] ; then
                                installerVer=$newStr
                                break
                        else
                                oldStr=$newStr
                        fi
                done

                echo Upgrading the version in the /etc/PMC-Sierra/maxRAID.conf file >> $installLogFile
                sed 's/'$installedVer'/'$installerVer'/' </etc/PMC-Sierra/maxRAID.conf >/etc/PMC-Sierra/maxRAID.conf.new
                rm -rf /etc/PMC-Sierra/maxRAID.conf
                mv /etc/PMC-Sierra/maxRAID.conf.new /etc/PMC-Sierra/maxRAID.conf

                echo Product is upgraded successfully.
		echo Product is upgraded successfully in silent mode >> $installLogFile
        fi

fi
